// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "../Asset.hpp"
#include "../AssetLoader.hpp"
#include "display/Texture.hpp"

namespace Dawn {
  class TextureAsset : public Asset {
    protected:
      AssetLoader loader;
      uint8_t *buffer = nullptr;
      int32_t width = -1, height = -1;
      struct Color *colors;

    public:
      Texture texture;

      /**
       * Constructs a texture asset loader. You should instead use the parent
       * asset managers' abstracted load method
       * 
       * @param assetManager Asset manager this asset belongs to.
       * @param name File name asset to load, omitting the extension.
       */
      TextureAsset(AssetManager *assetManager, std::string name);

      void updateSync() override;
      void updateAsync() override;

      /**
       * Dispose / Cleanup the texture asset. Will also dispose the underlying
       * texture itself.
       */
      ~TextureAsset();
  };
}