// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "Camera.hpp"

namespace Dawn {
  class PixelPerfectCamera : public SceneItemComponent {
    protected:
      Camera *camera = nullptr;
      /** Event for when the render target is resized. */
      void onRenderTargetResized(float_t w, float_t h);

      /**
       * Updates the underlying camera's projection information.
       */
      void updateDimensions();

    public:
      float_t scale = 1.0f;

      /**
       * Create a new PixelPerfectCamera Component.
       * 
       * @param item Item that this component belongs to.
       */
      PixelPerfectCamera(SceneItem *item);
      
      std::vector<SceneItemComponent*> getDependencies() override;
      void onStart() override;
      void onDispose() override;
  };
}