// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/components/display/MeshHost.hpp"
#include "display/RenderTarget.hpp"
#include "display/mesh/QuadMesh.hpp"

namespace Dawn {
  class SimpleRenderTargetQuad : public SceneItemComponent {
    protected:
      MeshHost *meshHost = nullptr;
      RenderTarget *renderTarget = nullptr;

      void onRenderTargetResized(RenderTarget *target, float_t w, float_t h);

    public:
      /**
       * Creates a SimpleRenderTargetQuad scene item component. This component
       * will update the attached MeshHost any time the render target provided
       * is updated / resized.
       * 
       * @param item Item that this component is attached to.
       */
      SimpleRenderTargetQuad(SceneItem *item);

      /**
       * Sets the render target to use for this quad. Can be set to nullptr when
       * you no longer wish to listen for resize events.
       * 
       * @param rt Render target to attach to.
       */
      void setRenderTarget(RenderTarget *rt);
      
      std::vector<SceneItemComponent*> getDependencies() override;
      void onStart() override;
      void onDispose() override;
  };
}