// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TicTacToeGame.hpp"
#include "game/DawnGame.hpp"
#include "scene/components/example/ExampleSpin.hpp"
#include "scene/components/physics/3d/CubeCollider.hpp"

using namespace Dawn;

TicTacToeGame::TicTacToeGame(SceneItem *item) : SceneItemComponent(item) {}

void TicTacToeGame::onStart() {
  // Map tiles by tile number = tile
  auto tiles = getScene()->findComponents<TicTacToeTile>();
  auto itTiles = tiles.begin();
  while(itTiles != tiles.end()) {
    this->tiles[(*itTiles)->tile] = *itTiles;
    ++itTiles;
  }

  // Listen
  getScene()->eventSceneUpdate.addListener(this, &TicTacToeGame::onSceneUpdate);
}

void TicTacToeGame::onDispose() {
  getScene()->eventSceneUpdate.removeListener(this, &TicTacToeGame::onSceneUpdate);
}

void TicTacToeGame::onSceneUpdate() {
  // Get mouse in screen space.
  auto mouse = getGame()->inputManager.getAxis2D(INPUT_BIND_MOUSE_X, INPUT_BIND_MOUSE_Y);
  mouse *= 2.0f;
  mouse -= glm::vec2(1, 1);

  Camera *camera = getScene()->findComponent<Camera>();
  if(camera == nullptr) return;

  struct Ray3D ray;
  ray.origin = camera->transform->getWorldPosition();
  ray.direction = camera->getRayDirectionFromScreenSpace(mouse);
  

  // Find the hovered tile (if any)
  TicTacToeTile *hovered = nullptr;
  auto results = getPhysics()->raycast3DAll(ray);
  auto itResult = results.begin();
  while(itResult != results.end()) {
    auto result = *itResult;
    auto tile = result.collider->item->getComponent<TicTacToeTile>();
    if(tile == nullptr) {
      ++itResult;
      continue;
    }
    
    hovered = tile;
    break;
  }

  // Now update the state of each tile, also get the state while we are at it
  std::map<uint8_t, enum TicTacToeTileState> tileMap;

  auto itTiles = tiles.begin();
  while(itTiles != tiles.end()) {
    auto t = itTiles->second;

    if(t == hovered) {
      if(t->tileState == TIC_TAC_TOE_EMPTY && getGame()->inputManager.isPressed(INPUT_BIND_MOUSE_CLICK)) {
        t->tileState = nextMove;
        nextMove = nextMove == TIC_TAC_TOE_NOUGHT ? TIC_TAC_TOE_CROSS : TIC_TAC_TOE_NOUGHT;
      } else if(t->tileState == TIC_TAC_TOE_EMPTY) {
        t->hovered = true;
      }
    } else {
      t->hovered = false;
    }
    tileMap[itTiles->second->tile] = itTiles->second->tileState;
    ++itTiles;
  }

  // Determine winner
  std::vector<uint8_t> winningCombo;
  auto winner = ticTacToeDetermineWinner(tileMap, &winningCombo);
}